@@ -3,12 +3,17 @@
 
 #define IDX(i,j,ld) ((((j)-1) * (ld))+((i)-1))
 void ipi_arg_max(double *res, int *n, int *ipargmax, double *ipmax){
-	/*consider a version with median!*/
-	int i;
+
+	int i,k;
+	int max_count;
 	double tmp,max_fabs;
+	
+	
+
+
 
 	ipargmax[0] = 0;
-	max_fabs = 0;
+	max_fabs = -1;
 
 	for(i=0; i < (*n)-1; i++){
 		tmp = fabs(res[i]);
@@ -16,9 +21,25 @@ void ipi_arg_max(double *res, int *n, int *ipargmax, double *ipmax){
 		if(tmp>max_fabs){
 			ipargmax[0] = i;
 			max_fabs = tmp;
-		}
+			max_count = 1;
+		}else if(tmp==max_fabs) max_count++;
 	}
+	
+	/*if there are multiple points maximizing cusums, we take the median*/
 
+	if(max_count>1){
+		max_count = max_count/2 + (int)(max_count%2);
+		
+		k=0;
+		i=0;
+
+		while((i< ((*n)-1))  && (k<max_count)){
+			i++;
+			if(fabs(res[i])==max_fabs) k++;
+		}
+
+		ipargmax[0] = i;
+	}
 
 		
 
@@ -63,7 +84,7 @@ void wbs_ipi(double *x, int *n, double *res, double *iplus, double *iminus, int
 
 
 
-void bs_rec(double *x, int *n, int s, int e, int *len, double *res,double *iplus, double *iminus, double *ipres,int *ipargmax, double *ipmax, double minth){
+void bs_rec(double *x, int *n, int s, int e, int *len, double *res,double *iplus, double *iminus, double *ipres,int *ipargmax, double *ipmax, double minth, int scale){
 
 	*len = e-s+1;
 	int cptcand;
@@ -85,9 +106,10 @@ void bs_rec(double *x, int *n, int s, int e, int *len, double *res,double *iplus
 		}
 
 		res[IDX(cptcand,5,*n-1)] = minth;
+		res[IDX(cptcand,6,*n-1)] = (double)scale;
 
-		bs_rec(x, n, s, cptcand, len, res,iplus, iminus,ipres,ipargmax, ipmax, minth);
-		bs_rec(x, n, cptcand+1, e, len, res,iplus, iminus,ipres,ipargmax, ipmax, minth);
+		bs_rec(x, n, s, cptcand, len, res,iplus, iminus,ipres,ipargmax, ipmax, minth, scale+1);
+		bs_rec(x, n, cptcand+1, e, len, res,iplus, iminus,ipres,ipargmax, ipmax, minth, scale+1);
 
 	}
 
@@ -102,7 +124,7 @@ void bs_rec_wrapper(double *x, int *n, double *res){
 	int *ipargmax = Calloc(1,int);
 	int *len = Calloc(1,int);
 	/*negative value of minth serves as infinity*/
-	bs_rec(x, n, 1, *n, len, res,iplus, iminus,ipres,ipargmax, ipmax, -1.0);
+	bs_rec(x, n, 1, *n, len, res,iplus, iminus,ipres,ipargmax, ipmax, -1.0, 1);
 
 	Free(iplus);
 	Free(iminus);
@@ -112,7 +134,7 @@ void bs_rec_wrapper(double *x, int *n, double *res){
 	Free(len);
 }
 
-void wbs_int_rec(double *x, int *n, int s, int e, int *len, double *res, double *iplus, double *iminus, double *ipres, int *ipargmax, double *ipmax, double *wbsres, int *index, int indexn, int *M, double minth){
+void wbs_int_rec(double *x, int *n, int s, int e, int *len, double *res, double *iplus, double *iminus, double *ipres, int *ipargmax, double *ipmax, double *wbsres, int *index, int indexn, int *M, double minth, int scale){
 	*len = e-s+1;
 
 	if(*len>1){
@@ -130,7 +152,7 @@ void wbs_int_rec(double *x, int *n, int s, int e, int *len, double *res, double
 				if(minth>wbsres[IDX(index[0],5,*M)] || minth < 0){
 					minth = wbsres[IDX(index[0],5,*M)];
 				}
-
+				
 			}else{
 				cptcand= *ipargmax +s;
 				res[IDX(cptcand,1,*n-1)] = (double)s;
@@ -144,6 +166,7 @@ void wbs_int_rec(double *x, int *n, int s, int e, int *len, double *res, double
 			}
 
 			res[IDX(cptcand,5,*n-1)] = minth;
+			res[IDX(cptcand,6,*n-1)] = (double)scale;
 
 			/*left*/
 			int *indexl= Calloc(indexn,int);
@@ -165,28 +188,28 @@ void wbs_int_rec(double *x, int *n, int s, int e, int *len, double *res, double
 			if(indexnl){
 				indexl = Realloc(indexl,indexnl,int);
 	
-				wbs_int_rec(x, n, s, cptcand, len, res, iplus, iminus, ipres, ipargmax, ipmax, wbsres, indexl, indexnl, M,minth);
+				wbs_int_rec(x, n, s, cptcand, len, res, iplus, iminus, ipres, ipargmax, ipmax, wbsres, indexl, indexnl, M,minth, scale+1);
 				Free(indexl);
 			}else{
 				Free(indexl);
 
-				bs_rec(x, n, s, cptcand, len, res,iplus, iminus,ipres,ipargmax, ipmax,minth);
+				bs_rec(x, n, s, cptcand, len, res,iplus, iminus,ipres,ipargmax, ipmax,minth, scale+1);
 			}
 
 			if(indexnr){
 				indexr = Realloc(indexr,indexnr,int);
 		
-				wbs_int_rec(x, n, cptcand +1, e, len, res, iplus, iminus, ipres, ipargmax, ipmax, wbsres, indexr, indexnr, M,minth);
+				wbs_int_rec(x, n, cptcand +1, e, len, res, iplus, iminus, ipres, ipargmax, ipmax, wbsres, indexr, indexnr, M,minth,scale+1);
 				Free(indexr);
 			}else{
 				Free(indexr);
 		
-				bs_rec(x, n, cptcand +1, e, len, res,iplus, iminus,ipres,ipargmax, ipmax,minth);
+				bs_rec(x, n, cptcand +1, e, len, res,iplus, iminus,ipres,ipargmax, ipmax,minth,scale+1);
 			}
 			
 		}else{
 	
-			bs_rec(x, n, s, e, len, res,iplus, iminus,ipres,ipargmax, ipmax,minth);
+			bs_rec(x, n, s, e, len, res,iplus, iminus,ipres,ipargmax, ipmax,minth, scale);
 		}
 	}
 }
@@ -231,7 +254,7 @@ void wbs_int_rec_wrapper(double *x, int *n, double *res,  int *intervals, int *M
 
 	/*standard wbs part*/
 	
-	wbs_int_rec(x, n, 1, *n, len, res, iplus, iminus, ipres, ipargmax, ipmax, wbsres, index, *M, M,-1.0);
+	wbs_int_rec(x, n, 1, *n, len, res, iplus, iminus, ipres, ipargmax, ipmax, wbsres, index, *M, M,-1.0,1);
 
 	Free(iplus);
 	Free(iminus);
@@ -244,7 +267,110 @@ void wbs_int_rec_wrapper(double *x, int *n, double *res,  int *intervals, int *M
 }
 
 
+void wbs_rec(double *x, int *n, int s, int e, int *len, double *res, double *wbsres, int *index, int indexn, int *M, int scale){
+	
+        *len = e-s+1;
 
+	if(*len>1){
+		if(indexn > 0){
+			int cptcand;
+	
+			cptcand= (int) wbsres[IDX(index[0],3,*M)];
+	
+			res[IDX(cptcand,1,*n-1)] = (double)(wbsres[IDX(index[0],1,*M)]);
+			res[IDX(cptcand,2,*n-1)] = (double)(wbsres[IDX(index[0],2,*M)]);
+			res[IDX(cptcand,3,*n-1)] = (double)cptcand;
+			res[IDX(cptcand,4,*n-1)] = (double)(wbsres[IDX(index[0],4,*M)]);	
+			res[IDX(cptcand,5,*n-1)] = wbsres[IDX(index[0],5,*M)];
+			res[IDX(cptcand,6,*n-1)] = (double)scale;
+
+			/*left*/
+			int *indexl= Calloc(indexn,int);
+			int *indexr= Calloc(indexn,int);
+			int indexnl = 0;
+			int indexnr = 0;
+			int i;
+
+			for(i=1; i<=indexn; i++){
+				if((wbsres[IDX(index[i-1],1,*M)] >= s) & (wbsres[IDX(index[i-1],2,*M)]  <= cptcand) ){
+					indexl[indexnl] = index[i-1];
+					indexnl ++;
+				}else if((wbsres[IDX(index[i-1],1,*M)] >= (cptcand+1)) & (wbsres[IDX(index[i-1],2,*M)]  <= e) ){
+					indexr[indexnr] = index[i-1];
+					indexnr ++;
+				}
+			}
+
+			if(indexnl){
+				indexl = Realloc(indexl,indexnl,int);
+				wbs_rec(x, n, s, cptcand, len, res, wbsres, indexl, indexnl, M, scale+1);
+				Free(indexl);
+			}
+			if(indexnr){
+				indexr = Realloc(indexr,indexnr,int);
+				wbs_rec(x, n, cptcand +1, e, len, res,  wbsres, indexr, indexnr, M, scale+1);
+				Free(indexr);
+			}
+			
+		}
+	}
+}
+
+void wbs_rec_wrapper(double *x, int *n, double *res,  int *intervals, int *M){
+	double *iplus= Calloc(*n-1,double);
+	double *iminus= Calloc(*n-1,double);
+	double *ipres= Calloc(*n-1,double);
+	double *wbsres= Calloc(*M * 5,double);
+	double *ipmax = Calloc(1,double);
+	int *index = Calloc(*M,int);
+	int *ipargmax = Calloc(1,int);
+	int *len = Calloc(1,int);
+	int i,s,e,cptcand;
+	
+	/* find cpt candidates on given intervals*/
+
+	for(i=1; i<=*M;i++){
+		s = intervals[IDX(i,1,*M)];
+		e = intervals[IDX(i,2,*M)];
+		len[0] = e-s+1;
+
+		wbs_ipi(&x[s-1], len, ipres, iplus, iminus,ipargmax,ipmax);
+		cptcand= *ipargmax +s;
+
+		wbsres[IDX(i,1,*M)] = (double)s;
+		wbsres[IDX(i,2,*M)] = (double)e;
+		wbsres[IDX(i,3,*M)] = (double)cptcand;
+		wbsres[IDX(i,4,*M)] = (double)(*ipmax);
+		wbsres[IDX(i,5,*M)] = fabs(*ipmax);
+		index[i-1] = i;
+	}
+	/* sort elementd from the one with the largest abs(cusum)*/
+	double *tmp= Calloc(*M,double);
+
+	memcpy(tmp,&wbsres[IDX(1,5,*M)],*M * sizeof(double));
+
+	revsort(tmp,index,*M);
+
+	Free(tmp);
+	
+
+
+	/*standard wbs part*/
+	
+	wbs_rec(x, n, 1, *n, len, res, wbsres, index, *M, M, 1);
+
+	Free(iplus);
+	Free(iminus);
+	Free(ipres);
+	Free(ipmax);
+	Free(index);
+	Free(ipargmax);
+	Free(len);
+	Free(wbsres);
+}
+
+
+/*
 void wbs(double *x, int *n, double *res, int *intervals, int *M){
 	double *iplus= Calloc(*n-1,double);
 	double *iminus= Calloc(*n-1,double);
@@ -276,4 +402,5 @@ void wbs(double *x, int *n, double *res, int *intervals, int *M){
 	Free(ipargmax);
 	Free(len);
 }
+*/
 
